package hage;

#if !flash
import kha.input.Keyboard;
import kha.input.Mouse;
#else
import flash.events.KeyboardEvent;
import flash.Lib;
#end

import kha.Key;

import haxe.ds.Vector;
import haxe.ds.GenericStack;

/**
 * ...
 * @author JanGamesCo
 */
class HGEInput
{
	/**Keyboard*/
	private static inline var KEYS:UInt = 500;
	private static inline var UP:UInt = 0;
	private static inline var DOWN:UInt = 1;
	private static inline var PRESSED:UInt = 2;
	
	/*Key arrays, size 256*/
	private var keyVector:Vector<UInt>;
	/*Input queue*/
	private var inputQueue:GenericStack<Int>;
	
	/**Mouse vars*/
	public var leftClick:Bool;
	public var rightClick:Bool;
	public var mouseX(get, set):Int;
	public var mouseY(get, set):Int;
	public var mousePosition:HGEVector2;
	
	/*Init*/
	public function new():Void
	{
		//Init vars
		keyVector = new Vector(KEYS);
		for(i in 0...KEYS) {
			keyVector[i] = UP;
		}
		
		mousePosition = new HGEVector2();
		
		//Input queue to update pressed
		inputQueue = new GenericStack<Int>();
		
		//Init input listeners
		#if !flash
		Keyboard.get().notify(onKeyDown, onKeyUp);
		Mouse.get().notify(onMouseDown, onMouseUp, onMouseMove, onMouseWheel);
		#else
		Lib.current.stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
		Lib.current.stage.addEventListener(KeyboardEvent.KEY_UP, onKeyUp);
		#end
	}

	/*Check if key is up*/
	public function keyUp(KeyV:Int):Bool { return keyVector[KeyV] == UP; }

	/*Check if key is down*/
	public function keyDown(KeyV:Int):Bool { return keyVector[KeyV] >= DOWN; }

	/*Check if key is pressed*/
	public function keyPressed(KeyV:Int):Bool {	return keyVector[KeyV] == PRESSED; }
	
	/*Update input*/
	private var t = 0;
	public function update():Void
	{		
		while (!inputQueue.isEmpty()) {
			t = inputQueue.pop();
			keyVector[t] = keyVector[t] == PRESSED ? DOWN : UP;
		}
	}

	/*Clear the input values*/
	//TODO Fix the button getting stuck on focus lost
	public function clearInput():Void
	{
		for (i in 0...KEYS) {
			keyVector[i] = UP;
		}
		
		while (!inputQueue.isEmpty()) {
			inputQueue.pop();
		}
		
		leftClick = false;
		rightClick = false;
	}
	
	/*Get keycode*/
	private function getKeycode(KeyV:Key, Char:String):UInt
	{
		var code:UInt = 0;
		
		switch(KeyV) {
			case Key.CHAR:
				code = Char.toUpperCase().charCodeAt(0);
			case Key.LEFT:
				code = HGEKeys.LEFT;
			case Key.RIGHT:
				code = HGEKeys.RIGHT;
			case Key.UP:
				code = HGEKeys.UP;
			case Key.DOWN:
				code = HGEKeys.DOWN;
			default:
				
		}
		
		return code;
	}
	
	/*On key down event*/
	#if !flash
	private function onKeyDown(KeyV:Key, Char:String):Void
	{
		var code = getKeycode(KeyV, Char);
		keyVector[code] = PRESSED;
		inputQueue.add(code);
		
		#if debug
		//if (HGE.debug) { trace("" + KeyV + " " + Char + " " + code + Type.getClass(code)); }
		#end
	}
	#else
	private function onKeyDown(Ev:KeyboardEvent):Void
	{
		keyVector[Ev.keyCode] = PRESSED;
		inputQueue.add(Ev.keyCode);		
	}
	#end
	
	/*On key up event*/
	#if !flash
	private function onKeyUp(KeyV:Key, Char:String):Void
	{
		keyVector[getKeycode(KeyV, Char)] = UP;
	}
	#else
	private function onKeyUp(Ev:KeyboardEvent):Void
	{
		keyVector[Ev.keyCode] = UP;
	}
	#end	
	
	/*On mouse down*/
	private function onMouseDown(Val:Int, X:Int, Y:Int):Void
	{
		switch (Val)
		{
			case 0:
				leftClick = true;
			case 1:
				rightClick = true;
			default:
				
		}
	}
	
	/*On mouse up*/
	private function onMouseUp(Val:Int, X:Int, Y:Int):Void
	{
		switch (Val) 
		{
			case 0:
				leftClick = false;
			case 1:
				rightClick = false;
			default:
				
		}
	}
	
	/*On mouse move*/
	private function onMouseMove(X:Int, Y:Int):Void
	{
		mouseX = X;
		mouseY = Y;
	}
	
	/*On mouse wheel delta*/
	private function onMouseWheel(V:Int):Void
	{
		#if debug
		if (HGE.debug) { trace("onMouseWheel: " + V); }
		#end
	}
	
	/**Mouse getter and setter for X and Y*/
	private function get_mouseX():Int { return Math.floor(mousePosition.x); }
	private function set_mouseX(Value:Int):Int { return Math.floor(mousePosition.x = Value); }
	private function get_mouseY():Int { return Math.floor(mousePosition.y); }
	private function set_mouseY(Value:Int):Int { return Math.floor(mousePosition.y = Value); }
}