package hage;

/**
 * ...
 * @author JanGamesCo
 */
class HGEObject extends HGESprite
{
	/** Velocity, added to position each frame */
	public var velocity:HGEVector2;
	/** Acceleration, added to velocity each frame */
	public var acceleration:HGEVector2;
	/** Max object velocity */
	public var maxVelocity:HGEVector2;
	
	public var angularVelocity:Float;
	
	/** Attached object */
	private var attachedObject:HGESprite;
	private var attached:Bool;
	public var attachedOffset:HGEVector2;
	
	public var wordWrap:Bool;

	public function new(X:Float=0, Y:Float=0, Width:Float=0, Height:Float=0, ColorV:UInt=null, Alpha:Float=1) 
	{
		super(X, Y, Width, Height, ColorV, Alpha);
		
		//Attached object null
		attachedObject = null;
		attachedOffset = new HGEVector2();
		
		//Init flags
		wordWrap = false;
		attached = false;
		
		angularVelocity = 0;
		
		//Reset
		reset();
	}
	
	public function attach(Other:HGESprite, OffsetX:Float = 0, OffsetY:Float = 0)
	{
		attached = true;
		attachedObject = Other;
		attachedOffset.set(OffsetX, OffsetY);
	}
	
	public function detach()
	{
		attached = false;
		attachedObject = null;
	}
	
	override public function update():Void 
	{		
		super.update();
		
		angle += angularVelocity * HGE.delta;
		if (angle > Math.PI * 2) {
			angle = 0;
		}
		
		if (attached) {
			if (!attachedObject.exists) {
				detach();
			}
			else {
				x = attachedObject.x + attachedOffset.x;
				y = attachedObject.y + attachedOffset.y;
			}
		} else {
			x += velocity.x * HGE.delta;
			y += velocity.y * HGE.delta;
			
			velocity.x = Math.max(Math.min(velocity.x + acceleration.x, maxVelocity.x), -maxVelocity.x);
			velocity.y = Math.max(Math.min(velocity.y + acceleration.y, maxVelocity.y), -maxVelocity.y);
			
			if (wordWrap) {
				x = x % HGE.width;
				if (x < 0) { x += HGE.width; }
				y = y % HGE.height;
				if (y < 0) { y += HGE.height; }
			}
		}
	}
	
	override public function reset():Void 
	{		
		//Init velocity
		velocity = new HGEVector2(0, 0);
		maxVelocity = new HGEVector2(Math.POSITIVE_INFINITY, Math.POSITIVE_INFINITY);
		acceleration = new HGEVector2(0, 0);
		
		super.reset();
	}
	
	override public function destroy():Void 
	{
		super.destroy();
		
		attachedObject = null;
		attachedOffset = null;
		velocity = null;
		maxVelocity = null;
		acceleration = null;
	}
	
}