package hage;

import hage.HGEData.SpriteSide;
import hage.HGEData.SpriteType;
import kha.arrays.Float32Array;
import kha.Color;
import kha.graphics2.Graphics;
import kha.graphics2.GraphicsExtension;
import kha.math.FastMatrix3;

/**
 * ...
 * @author JanGamesCo
 */
class HGESprite extends HGEBasic
{
	/** Alpha */
	public var alpha(get, set):Float;
	/** Angle */
	public var angle(default, set):Float;
	/** Origin */
	public var origin:HGEVector2;
	/** Position */
	public var x(default, set):Float;
	public var lastX:Float;
	public var y(default, set):Float;
	public var lastY:Float;
	public var position:HGEVector2;
	/** Width */
	public var width:Float;
	/** Height */
	public var height:Float;
	/** ScaleX */
	public var scaleX(default, set):Float;
	/** ScaleY */
	public var scaleY(default, set):Float;
	
	/** Circle radius */
	public var radius:Float;
	
	/** Border */
	public var borderColor:Color;
	public var borderStrength:Float;
	
	/** Sprite type, what is being displayed */
	public var spritetype:SpriteType;
	
	/** HGE draw matrix */
	private var matrix:FastMatrix3;
	/** Sprite color */
	public var color:Color;
	
	/** Create a new sprite */
	public function new(X:Float=0, Y:Float=0, Width:Float=0, Height:Float=0, ColorV:UInt=null, Alpha:Float=1)
	{
		super();
		
		//Init matrix
		matrix = new FastMatrix3(1, 0, 0, 0, 1, 0, 0, 0, 1);
		
		//Set position
		position = new HGEVector2();
		x = X;
		y = Y;
		
		//Set width and height
		width = Width;
		height = Height;
		
		//Color
		color = Color.fromValue(ColorV);
		
		//Alpha
		alpha = Alpha;
		
		//Origin
		origin = new HGEVector2(0, 0);
		
		//Angle
		angle = 0;		
		
		//Set sprite type
		spritetype = SpriteType.RECTANGLE;
		
		//Border
		borderColor = Color.White;
		borderStrength = 0;
		
		//Scale
		scaleX = 1;
		scaleY = 1;
	}
	
	/***Public functions***/
	/**  */
	
	/**  */
	public function aabb(Other:HGESprite):Bool
	{		
		if (Other.exists && exists &&
			x - origin.x < Other.x + Other.width + borderStrength * 0.5 &&
			x - origin.x + width > Other.x &&
			y - origin.y < Other.y + Other.height + borderStrength * 0.5 &&
			y - origin.y + height > Other.y) {
			return true;
		}
		
		return false;
	}
	
	/**  */
	public function sidedaabb(Other:HGESprite):SpriteSide
	{		
		if (Other.exists && exists &&
			x - origin.x < Other.x + Other.width &&
			x - origin.x + width > Other.x &&
			y - origin.y < Other.y + Other.height  &&
			y - origin.y + height > Other.y) {
			
			var lx = lastX + width;
			var ly = lastY + height;
			
			if (lx < Other.x) {
				return SpriteSide.LEFT;
			} else if (lastX > Other.x + Other.width) {
				return SpriteSide.RIGHT;
			} else if (ly < Other.y) {
				return SpriteSide.UP;
			} else {
				return SpriteSide.DOWN;
			}
		}
		
		return SpriteSide.NONE;
	}
	
	/***Overrides***/
	/** Update method */
	override public function update():Void
	{
		super.update();
		
		//Get last x and y
		lastX = x;
		lastY = y;
	}
	
	/** Draw Method */
	override public function draw():Void
	{
		super.draw();
		
		var g:Graphics = HGE.backbuffer.g2;
		
		g.set_color(color);
		g.set_opacity(alpha);
		
		#if debug
		if (HGE.debug) {
			g.drawRect(x - origin.x, y - origin.y, width + borderStrength * 0.5, height + borderStrength * 0.5);
		}
		#end
		
		g.pushTransformation(matrix);
		
		switch (spritetype) {
			case SpriteType.NONE:
				
			case SpriteType.RECTANGLE:
				g.fillRect(-origin.x, -origin.y, width, height);
			case SpriteType.CIRCLE:
				GraphicsExtension.fillCircle(g, -origin.x + radius, -origin.y + radius, radius);
			case SpriteType.TRIANGLE:
				g.fillTriangle(-origin.x, -origin.y, -origin.x, -origin.y + height, -origin.x + width, -origin.y + height / 2);
			case SpriteType.LINE:
				g.drawLine(-origin.x, -origin.y, width, height);
			case SpriteType.IMAGE:
				//TODO Do image drawing
			default:
				
		}
		
		g.set_color(borderColor);
		if (borderStrength > 0) {
			switch (spritetype) {
				case SpriteType.NONE:
					
				case SpriteType.RECTANGLE:
					g.drawRect(-origin.x, -origin.y, width, height, borderStrength);
				case SpriteType.CIRCLE:
					GraphicsExtension.drawCircle(g, -origin.x + radius, -origin.y + radius, radius, borderStrength);
				case SpriteType.TRIANGLE:
					var ax = -origin.x;
					var ay = -origin.y;
					var bx = ax;
					var by = ay + height;
					var cx = ax + width;
					var cy = ay + height / 2;
					g.drawLine( ax, ay, bx, by, borderStrength);
					g.drawLine( ax, ay, cx, cy, borderStrength);
					g.drawLine( bx, by, cx, cy, borderStrength);
				case SpriteType.LINE:
					
				case SpriteType.IMAGE:
					//TODO Do image drawing
				default:
					
			}
		}
		g.popTransformation();
	}
	
	/***Getters and setters***/
	private function get_alpha():Float { return 0; }
	private function set_alpha(Value:Float):Float { return 0; }
	
	private function set_x(Value:Float):Float { return x = position.x = matrix._20 = Value; }
	private function set_y(Value:Float):Float { return y = position.y = matrix._21 = Value; }
	private function set_scaleX(Value:Float):Float { return scaleX = matrix._00 = Value; }
	private function set_scaleY(Value:Float):Float { return scaleY = matrix._11 = Value; }
	
	private function set_angle(Value:Float):Float {
		matrix._00 = Math.cos(Value) * scaleX;
		matrix._11 = matrix._00;
		matrix._10 = Math.sin(Value) * scaleY;
		matrix._01 = -matrix._10;
		return angle = Value;
	}
}