package;

import hage.HGE;
import hage.HGEData;
import hage.HGEData.SpriteType;
import hage.HGEData.TextAlign;
import hage.HGEGame;
import hage.HGEKeys;
import hage.HGEObject;
import hage.HGEText;
import kha.Color;
import kha.Configuration;
import kha.Font;
import kha.FontStyle;
import kha.Loader;
import kha.LoadingScreen;
import levels.Level01;
import levels.Level02;
import levels.Level03;
import levels.LevelBase;

class LDGame extends HGEGame
{
	private var title:HGEText;
	private var text:HGEText;
	private var text2:HGEText;
	private var text3:HGEText;
	
	public function new()
	{
		super("Ludum Dare 34");
	}

	//Game init event
	override function init(): Void
	{
		super.init();
		
		//Setup screen configuration
		Configuration.setScreen(new LoadingScreen());
		Loader.the.loadRoom("level", initLevel);
	}
	
	//Level init event
	override private function initLevel(): Void {
		super.initLevel();
		
		//Create data singleton
		new Data();
		
		//Data singleton
		Data.the.game = this;
		
		//Get font		
		var font:Font = Loader.the.loadFont("Arial", new FontStyle(false, false, false), 16);
		Data.the.arialFont = font;
		
		var font:Font = Loader.the.loadFont("Nokia", new FontStyle(false, false, false), 16);
		Data.the.nokiaFont = font;
		
		//Background
		for (i in 0...100) {
			var background:HGEObject = new HGEObject(Math.random() * HGE.width, Math.random() * HGE.height, 
				Math.random() * HGE.width, Math.random() * HGE.height,
				Color.fromValue(Math.floor(0x000000FF * Math.random())), 1);
			if (Math.random() < 0.2) {
				background.color = Color.White;
				background.color.A = Math.random() * 0.003 + 0.001;
			} else {
				background.color.A = Math.random() * 0.01 + 0.001;
			}
				
			background.wordWrap = true;
			background.origin.x = background.width / 2;
			background.origin.y = background.height / 2;
			background.velocity.set(Math.random() * 5, Math.random() * 5);
			background.angularVelocity = Math.random() * 0.05;
			Data.the.backGroup.add(background);
		}
		
		for (i in 0...200) {
			var background:HGEObject = new HGEObject(Math.random() * HGE.width, Math.random() * HGE.height, 0, 0,
				Color.fromValue(Math.floor(0x00FFFFFF * Math.random())), 1);
			if (Math.random() < 0.05) {
				background.radius = Math.random() * 3 + 0.1;
			} else {
				background.radius = Math.random() * 1 + 0.1;
			}
			background.spritetype = SpriteType.CIRCLE;
			
			background.color.A = Math.random() * 0.3 + 0.1;
			background.wordWrap = true;
			background.origin.x = background.width / 2;
			background.origin.y = background.height / 2;
			background.velocity.set(Math.random() * 2, Math.random() * 2);
			background.angularVelocity = Math.random() * 0.05;
			Data.the.backGroup.add(background);
		}
		
		//Level
		var level:LevelBase = new Level01();
		level.kill();
		Data.the.levels.push(level);
		var level:LevelBase = new Level02();
		level.kill();
		Data.the.levels.push(level);
		var level:LevelBase = new Level03();
		level.kill();
		Data.the.levels.push(level);
		
		text = new HGEText(10, 10, 0, Data.the.nokiaFont);
		text.text = "Press SPACE to start the game\nGame made by JanGames for the Ludum Dare 34 Compo";
		add(text);
		
		title = new HGEText(0, -64, 0, Data.the.nokiaFont);
		title.setAlign(TextAlign.CENTER, HGE.width, TextAlign.CENTER, HGE.height);
		title.text = "Space Warrior";
		add(title);
		
		text2 = new HGEText(0, 0, 0, Data.the.nokiaFont);
		text2.setAlign(TextAlign.CENTER, HGE.width, TextAlign.CENTER, HGE.height);
		text2.text = "A/LEFT to move left, D/RIGHT to move right\nSpaceship autoshoots, collide with enemies to grow\nand have fun";
		add(text2);
		
		text3 = new HGEText(0, 64, 0, Data.the.nokiaFont);
		text3.setAlign(TextAlign.CENTER, HGE.width, TextAlign.CENTER, HGE.height);
		text3.text = "";
		add(text3);
		
		//Player
		Data.the.player = Data.the.playerShipPool.create(HGE.width / 2, HGE.height - 48);
		
		//Set screen
		Configuration.setScreen(this);
	}
	
	public function nextLevel()
	{
		Data.the.level++;
		
		reset();		
		
		if (Data.the.level == 3) { gameWon(); return; }
		
		text3.revive();
		text3.color.A = 1;
		text3.text = "Level " + (Data.the.level + 1);
		
		Data.the.levels.members[Data.the.level].revive();
	}
	
	public function gameWon()
	{		
		text.revive();
		text.color.A = 1;
		text2.revive();
		text2.text = "Great Job! You defeat the evil empire!";
		text2.color.A = 1;
		
		Data.the.level = -1;
		
		reset();
		
		Data.the.playerShipPool.reset(); 
		Data.the.player.revive();
		Data.the.player.x = HGE.width / 2;
		Data.the.player.y = HGE.height - 48;
	}
	
	public function gameOver()
	{
		text.revive();
		text.color.A = 1;
		text2.revive();
		text2.color.A = 1;
		
		Data.the.level = -1;
		
		reset();
		
		Data.the.playerShipPool.reset(); 
		Data.the.player.revive();
		Data.the.player.x = HGE.width / 2;
		Data.the.player.y = HGE.height - 48;
	}
	
	public function reset() {
		for (l in Data.the.levels.members) {
			l.kill();
		}
		
		if (Data.the.level == -1) {
			Data.the.boss01Pool.reset();
			Data.the.boss02Pool.reset();
			Data.the.boss03Pool.reset();
		}
		
		Data.the.enemy01Pool.reset(); 
		Data.the.jointPool.reset();
		Data.the.enemyPool.reset();
		Data.the.playerPool.reset();
		Data.the.virtualYPos = 0;
	}
	
	//Update event
	override public function update()
	{
		#if debug
		if (HGE.input.keyPressed(HGEKeys.E)) {
			nextLevel();
		}
		if (HGE.input.keyPressed(HGEKeys.Z)) {
			gameOver();
		}
		#end
		
		if (!Data.the.player.exists) {
			gameOver();
		}
		
		if (HGE.input.keyPressed(HGEKeys.SPACE) && (Data.the.level < 0 || Data.the.level == 3)) {
			nextLevel();
			Data.the.gameover = false;
		}
		
		if (Data.the.level >= 0 && Data.the.level < 3) {
			title.color.A = Math.max(title.color.A - HGE.delta, 0);
			text.color.A = Math.max(text.color.A - HGE.delta, 0);
			text2.color.A = Math.max(text2.color.A - HGE.delta, 0);
		}
		
		text3.color.A = Math.max(text3.color.A - HGE.delta / 10, 0);
		
		super.update();		
	}
	
}