package levels;

import hage.HGE;
import hage.HGEBasic;
import hage.HGEVector2;
import ships.Boss01;
import ships.Boss02;
import ships.Boss03;
import ships.Enemy01;
import ships.EnemyBase;

enum SpawnPattern {
	Unit ( x:Float, y:Float );
	Line ( x:Float, y:Float, count:Int, direction:HGEVector2);
	Rectangle ( x:Float, y:Float, countH:Int, countV:Int, directionH:HGEVector2, directionV:HGEVector2);
	Circle ( x:Float, y:Float, count:Int, startAngle:Float, deltaAngle:Float, startRadius:Float, deltaRadius:Float, VMultipier:Float );
}

typedef SpawnData = {vy:Float, x:Float, y:Float, enemy:Int, velX:Float, velY:Float, pattern:SpawnPattern}

/**
 * ...
 * @author JanGamesCo
 */
class LevelBase extends HGEBasic
{
	private var nextSpawn:Int;
	private var spawnList:Array<SpawnData>;

	public function new() 
	{
		super();
		
		spawnList = new Array();
		nextSpawn = 0;
	}
	
	override public function update():Void 
	{
		super.update();
		
		while (spawnList[nextSpawn] != null && Data.the.virtualYPos > spawnList[nextSpawn].vy) {
			spawnEnemies(spawnList[nextSpawn]);
			nextSpawn++;
		}
	}
	
	public function spawnEnemies(Dat:SpawnData)
	{
		var sx:Float = Dat.x;
		var sy:Float = Dat.y;
		
		var enemy:EnemyBase;
		switch (Dat.pattern.getName()) 
		{
			case "Unit":
				spawn(sx, sy, Dat.enemy, Dat.velX, Dat.velY);
			case "Line":
				var array:Array<Dynamic> = Dat.pattern.getParameters();
				sx += array[0];
				sy += array[1];
				var vector:HGEVector2 = array[3];
				for (i in 0...array[2]) {
					spawn(sx, sy, Dat.enemy, Dat.velX, Dat.velY);
					sx += vector.x;
					sy += vector.y;
				}
			case "Rectangle":
				var array:Array<Dynamic> = Dat.pattern.getParameters();
				sx += array[0];
				sy += array[1];
				var vectorH:HGEVector2 = array[4];
				var vectorV:HGEVector2 = array[5];
				for (i in 0...array[3]) {
					var t = sx;
					for (j in 0...array[2]) {
						spawn(sx, sy, Dat.enemy, Dat.velX, Dat.velY);
						sx += vectorH.x;
						sy += vectorH.y;
					}
					sx = t;
					
					sx += vectorV.x;
					sy += vectorV.y;
				}
			case "Circle":
				var array:Array<Dynamic> = Dat.pattern.getParameters();
				sx += array[0];
				sy += array[1];
				var angle = array[3];
				var radius = array[5];
				for (i in 0...array[2]) {
					spawn(sx + Math.cos(angle) * radius, sy + Math.sin(angle) * radius * array[7], Dat.enemy, Dat.velX, Dat.velY);
					angle += array[4];
					radius += array[6];
				}
			default:
				
		}
	}
	
	public function spawn(X:Float, Y:Float, Enemy:Int, VelocityX:Float, VelocityY:Float)
	{
		var enemy:EnemyBase;
		
		switch (Enemy) 
		{
			case 1:
				enemy = Data.the.enemy01Pool.create(X, Y, Enemy01Type.Square);
			case 2:
				enemy = Data.the.enemy01Pool.create(X, Y, Enemy01Type.Triangle);
			case 3:
				enemy = Data.the.enemy01Pool.create(X, Y, Enemy01Type.Circle);
			case 10:
				enemy = Data.the.boss01Pool.create(X, Y);
			case 100:
				enemy = Data.the.boss01Pool.create(X, Y);
				cast(enemy, Boss01).isBoss = true;
			case 101:
				enemy = Data.the.boss02Pool.create(X, Y);
				cast(enemy, Boss02).isBoss = true;
			case 102:
				enemy = Data.the.boss03Pool.create(X, Y);
				cast(enemy, Boss03).isBoss = true;				
			default:
				return;
		}
		
		enemy.velocity.add(new HGEVector2(VelocityX, VelocityY));
		
		return;
	}
	
	override public function revive():Void 
	{
		super.revive();
		
		nextSpawn = 0;
	}
	
}