package ships;

import hage.HGE;
import hage.HGEObject;
import hage.HGEData.SpriteType;
import kha.audio1.Audio;
import kha.Color;
import kha.Loader;

/**
 * ...
 * @author JanGamesCo
 */
class EnemyBase extends HGEObject
{
	private var shootTimer:Float;
	private var shootRate:Float;
	
	public var growth:Float;
	public var health:Float;
	
	public var eaten:Bool;
	public var activated:Bool;

	public function new(X:Float=0, Y:Float=0, Width:Float = 0, Height:Float = 0) 
	{
		super(X, Y, Width, Height, 0xFFFF0000, 1);
		
		//Player growth
		growth = 1;
		
		//Timers and firerates
		shootRate = 3;
		
		//Reset
		reset();
	}
	
	override public function update():Void 
	{
		super.update();
		
		//Shooting
		shootTimer -= HGE.delta;
		if (shootTimer < 0 && y > 0 && y < 360) {
			shoot();
			shootTimer = shootRate;
		}
		
		//Player bullet collision
		for (bullet in Data.the.playerPool.members) {
			if (aabb(bullet)) {
				health -= bullet.damage;
				bullet.kill();
				if (health < 0) {
					kill();
				}
			}
		}
		
		//Destroy if offscreen
		if (x < -HGE.width || x > HGE.width * 2 || y - height * 2 > HGE.height) {
			kill();
		}
	}
	
	override public function revive():Void 
	{
		super.revive();
		
		//Velocity
		velocity.set(0, Data.the.playerSpeed);
		angularVelocity = 0;
		
		angle = 0;
		
		//Health
		health = 1;
		
		//Timer is 0
		shootTimer = 0;
		
		//
		eaten = false;
		activated = true;
	}
	
	public function shoot():Void
	{
		var audio = Audio.playSound(Data.the.shoot2);
		if(audio != null) { audio.volume = 0.005; }
	}
	
	override public function kill():Void 
	{
		super.kill();
		
		var audio;
		if (eaten) {
			audio = Audio.playSound(Data.the.powerup1);			
		} else {
			if(Math.random() < 0.5) {
				audio = Audio.playSound(Data.the.explosion1);
			} else {
				audio = Audio.playSound(Data.the.explosion2);
			}
		}
		
		if(audio != null) { audio.volume = 0.005; }
	}
	
	override public function draw():Void 
	{		
		super.draw();
		
		//var g = HGE.backbuffer.g2;
		//
		//g.set_color(Color.Orange);
		//g.pushTransformation(matrix);
		//var f:Float = Math.sin(HGE.lastTime * 5 + ID) * 0.05;
		//g.fillTriangle(1, -1, width, -1, width / 2, -height * (0.3 + f));
		//g.popTransformation();
	}
	
}