package ships;

import hage.HGE;
import hage.HGEKeys;
import hage.HGEObject;
import hage.HGEData.SpriteType;
import kha.audio1.Audio;
import kha.Color;
import kha.Loader;

/**
 * ...
 * @author JanGamesCo
 */
class Player extends HGEObject
{
	private var shootTimer:Float;
	private var shootRate:Float;
	
	private var playerSpeed:Float;

	public function new(X:Float=0, Y:Float=0) 
	{
		super(X, Y, 16, 16, 0xFFFFFFFF, 1);
		
		//Player triangle looking up
		spritetype = SpriteType.TRIANGLE;
		angle = Math.PI * 0.5;
		
		origin.x = 8;
		origin.y = 8;
		
		borderColor = Color.fromValue(0xFFAAAAFF);
		borderStrength = 2;
		
		//Shot rate
		shootRate = 1.5;
		shootTimer = shootRate;
		
		playerSpeed = Data.the.playerSpeed;
	}
	
	override public function update():Void 
	{
		super.update();
		
		if (attached) {
			attachedOffset.rotate(HGE.delta);
		} else if(attachedOffset.x == 0) {
			//Virtual y movement
			Data.the.virtualYPos += playerSpeed * HGE.delta;
		}		
		
		//Shots
		shootTimer -= HGE.delta;
		if (shootTimer < 0 && y > 0) {
			Data.the.playerPool.fire(x - width / 4 - borderStrength / 2, y - 16);
			var audio = Audio.playSound(Data.the.shoot1);
			if(audio != null) { audio.volume = 0.005; }
			shootTimer = shootRate;
		}
		
		x = Math.max(Math.min(x, HGE.width - width), width);
		
		//Movement
		velocity.set(0, 0);
		if (x > 0 && HGE.input.keyDown(HGEKeys.A) || HGE.input.keyDown(HGEKeys.LEFT)) {
			velocity.set(-100, 0);
		} else if (x < HGE.width - width && HGE.input.keyDown(HGEKeys.D) || HGE.input.keyDown(HGEKeys.RIGHT)) {
			velocity.set(100, 0);
		}
		
		//Collision with enemy bullets
		for (bullet in Data.the.enemyPool.members) {
			if (aabb(bullet)) {
				healDamage( -bullet.damage);
				bullet.kill();
			}
		}
		
		//Collision with enemies
		for (enemy in Data.the.enemy01Pool.members) {
			if (aabb(enemy)) {
				healDamage(enemy.growth);
				enemy.eaten = true;
				enemy.kill();
			}
		}
		
		//Death
		if (width < 12) {
			kill();
		}
		
		//Split		
		if (width > 20) {
			var player:Player = Data.the.playerShipPool.create(x + 30, y);
			player.attach(this, 30, 0);
			player.attachedOffset.rotate(Math.random() * Math.PI * 2);
			width -= 6;
			height -= 6;
		}
		
		#if debug
		if (HGE.input.keyPressed(HGEKeys.Q)) {
			width = 21;
			height = 21;
		}
		#end
	}
	
	override public function detach() 
	{		
		super.detach();
	}
	
	override public function kill():Void 
	{
		super.kill();
		
		if (ID == Data.the.player.ID && !Data.the.gameover) {
			Data.the.gameover = true;
			Data.the.game.gameOver();
		}
	}
	
	override public function revive():Void 
	{
		super.revive();
	
		width = 16;
		height = 16;
	}
	
	public function healDamage(Delta:Float) {
		width += Delta;
		origin.x = width / 2;
		height += Delta;
		origin.y = height / 2;		
	}
	
}